package com.ruoyi.inventory.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.ruoyi.asset.domain.TbAssetInformation;
import com.ruoyi.asset.domain.TbLocation;
import com.ruoyi.asset.mapper.TbAssetInformationMapper;
import com.ruoyi.asset.mapper.TbLocationMapper;
import com.ruoyi.inventory.domain.TagInfo;
import com.ruoyi.inventory.domain.dto.TakeStockDTO;
import com.ruoyi.inventory.domain.dto.TbAssetInventoryDTO;
import com.ruoyi.utils.IDUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.inventory.mapper.TbAssetInventoryMapper;
import com.ruoyi.inventory.domain.TbAssetInventory;
import com.ruoyi.inventory.service.ITbAssetInventoryService;

/**
 * 资产盘点记录Service业务层处理
 * 
 * @author ydl
 * @date 2023-06-01
 */
@Service
public class TbAssetInventoryServiceImpl implements ITbAssetInventoryService
{
    @Autowired
    private TbAssetInventoryMapper tbAssetInventoryMapper;

    @Autowired
    private TbAssetInformationMapper informationMapper;

    @Autowired
    private TbLocationMapper locationMapper;

    /**
     * 查询资产盘点记录
     * 
     * @param id 资产盘点记录主键
     * @return 资产盘点记录
     */
    @Override
    public TbAssetInventory selectTbAssetInventoryById(Long id)
    {
        return tbAssetInventoryMapper.selectTbAssetInventoryById(id);
    }

    /**
     * 查询资产盘点记录列表
     * 
     * @param tbAssetInventory 资产盘点记录
     * @return 资产盘点记录
     */
    @Override
    public List<TbAssetInventory> selectTbAssetInventoryList(TbAssetInventory tbAssetInventory)
    {
        return tbAssetInventoryMapper.selectTbAssetInventoryList(tbAssetInventory);
    }

    /**
     * 新增资产盘点记录
     * 
     * @param tbAssetInventory 资产盘点记录
     * @return 结果
     */
    @Override
    public int insertTbAssetInventory(TbAssetInventory tbAssetInventory)
    {
        tbAssetInventory.setOrderNumber(IDUtil.getUUID1());
        Long id = Long.parseLong(tbAssetInventory.getInventoryLocation());
        TbLocation location = locationMapper.selectTbLocationById(id);
        tbAssetInventory.setInventoryLocation(location.getNumber());
        return tbAssetInventoryMapper.insertTbAssetInventory(tbAssetInventory);
    }

    //手持机盘点
    @Override
    public int addByTakeStock(TakeStockDTO dto){
        //解析实体
        List<TagInfo> list = dto.getAssetList();
        String location = dto.getInventoryLocation();
        String orderNumber = dto.getOrderNumber();
        //提取epc
        ArrayList<String> epcList = new ArrayList<>();
        for (TagInfo tagInfo : list){
            epcList.add(tagInfo.getEpc());
        }
        //获取所在位置内的资产
        TbAssetInformation information = new TbAssetInformation();
        information.setLocationNumber(location);
        List<TbAssetInformation> informations = informationMapper.selectTbAssetInformationList(information);
        //提取rfid
        ArrayList<String> rfidList = new ArrayList<>();
        for (TbAssetInformation info : informations){
            rfidList.add(info.getNumber());
        }
        //对比盘点
        List<String> intersection = new ArrayList<>(rfidList);
        intersection.retainAll(epcList);
        if (intersection.size()<informations.size()){
            //盘亏
            if (!orderNumber.isEmpty()){
                TbAssetInventory inventory = tbAssetInventoryMapper.selectTbAssetInventoryByNumber(orderNumber);
                if (inventory.getRecordStatus() == 1L) {
                    inventory.setInventoryStatus(2L);
                    inventory.setInventoryResult(2L);
                    return updateTbAssetInventory(inventory);
                }else {
                    return 0;
                }
            }else {
                TbAssetInventory inventory = new TbAssetInventory();
                inventory.setInventoryStatus(2L);
                inventory.setInventoryResult(2L);
                inventory.setRecordStatus(1L);
                inventory.setInventoryLocation(location);
                inventory.setInventoryDate(new Date());
                inventory.setName("线下自测");
                return insertTbAssetInventory(inventory);
            }
        }else if (list.size()>intersection.size()){
            //盘盈
            if (!orderNumber.isEmpty()){
                TbAssetInventory inventory = tbAssetInventoryMapper.selectTbAssetInventoryByNumber(orderNumber);
                if (inventory.getRecordStatus() == 1L) {
                    inventory.setInventoryStatus(2L);
                    inventory.setInventoryResult(1L);
                    return updateTbAssetInventory(inventory);
                }else {
                    return 0;
                }
            }else {
                TbAssetInventory inventory = new TbAssetInventory();
                inventory.setInventoryStatus(2L);
                inventory.setInventoryResult(1L);
                inventory.setRecordStatus(1L);
                inventory.setInventoryLocation(location);
                inventory.setInventoryDate(new Date());
                inventory.setName("线下自测");
                return insertTbAssetInventory(inventory);
            }
        }else if (intersection.size()==informations.size()){
            //正常
            if (!orderNumber.isEmpty()){
                TbAssetInventory inventory = tbAssetInventoryMapper.selectTbAssetInventoryByNumber(orderNumber);
                if (inventory.getRecordStatus() == 1L) {
                    inventory.setInventoryStatus(2L);
                    inventory.setInventoryResult(3L);
                    return updateTbAssetInventory(inventory);
                }else {
                    return 0;
                }
            }else {
                TbAssetInventory inventory = new TbAssetInventory();
                inventory.setInventoryStatus(2L);
                inventory.setInventoryResult(3L);
                inventory.setRecordStatus(1L);
                inventory.setInventoryLocation(location);
                inventory.setInventoryDate(new Date());
                inventory.setName("线下自测");
                return insertTbAssetInventory(inventory);
            }
        }else {
            return 0;
        }
    }

    /**
     * 修改资产盘点记录
     * 
     * @param tbAssetInventory 资产盘点记录
     * @return 结果
     */
    @Override
    public int updateTbAssetInventory(TbAssetInventory tbAssetInventory)
    {
        return tbAssetInventoryMapper.updateTbAssetInventory(tbAssetInventory);
    }

    /**
     * 批量删除资产盘点记录
     * 
     * @param ids 需要删除的资产盘点记录主键
     * @return 结果
     */
    @Override
    public int deleteTbAssetInventoryByIds(Long[] ids)
    {
        return tbAssetInventoryMapper.deleteTbAssetInventoryByIds(ids);
    }

    /**
     * 删除资产盘点记录信息
     * 
     * @param id 资产盘点记录主键
     * @return 结果
     */
    @Override
    public int deleteTbAssetInventoryById(Long id)
    {
        return tbAssetInventoryMapper.deleteTbAssetInventoryById(id);
    }

    @Override
    public List<TbAssetInventoryDTO> selectTbAssetInventoryDTOList(TbAssetInventoryDTO dto) {
        return tbAssetInventoryMapper.selectTbAssetInventoryDTOList(dto);
    }
}
